@@ -1,4 +1,4 @@
-# Authen::TOTP version 0.0.3
+# Authen::TOTP version 0.0.6
 #
 # Copyright (c) 2020 Thanos Chatziathanassiou <tchatzi@arx.net>. All rights reserved.
 # This program is free software; you can redistribute it and/or
@@ -12,7 +12,7 @@ use vars qw(@ISA @EXPORT @EXPORT_OK);
 
 @EXPORT_OK = qw();
 
-$Authen::TOTP::VERSION='0.0.3';
+$Authen::TOTP::VERSION='0.0.6';
 $Authen::TOTP::ver=$Authen::TOTP::VERSION;
 
 use strict;
@@ -21,8 +21,6 @@ use utf8;
 use Carp;
 use Data::Dumper;
 
-use Digest::HMAC_SHA1();
-
 sub debug_print {
 	my $self = shift;
 	
@@ -141,6 +139,46 @@ sub secret {
 	my $self = shift;
 	return $self->{secret};
 }
+sub algorithm {
+	my $self = shift;
+	my $algorithm = shift;
+	$self->valid_algorithm($algorithm);
+
+	return $self->{algorithm};
+}
+
+sub hmac {
+	my $self = shift;
+	my $Td = shift;
+
+	if  ((eval {require Digest::SHA;1;} || 0) ne 1) {
+		# if module can't load
+		require Digest::SHA::PurePerl;
+		$self->{DEBUG} and $self->debug_print("Digest::SHA unavailable, using Digest::SHA::PurePerl()");
+		if ($self->{algorithm} eq 'SHA512') {
+			return Digest::SHA::PurePerl::hmac_sha512_hex($Td, $self->{secret});
+		}
+		elsif ($self->{algorithm} eq 'SHA256') {
+			return Digest::SHA::PurePerl::hmac_sha256_hex($Td, $self->{secret} );
+		}
+		else {
+			return Digest::SHA::PurePerl::hmac_sha1_hex($Td, $self->{secret} );
+		}
+	}
+	else {
+		#we have XS!
+		$self->{DEBUG} and $self->debug_print("using Digest::SHA()");
+		if ($self->{algorithm} eq 'SHA512') {
+			return Digest::SHA::hmac_sha512_hex($Td, $self->{secret});
+		}
+		elsif ($self->{algorithm} eq 'SHA256') {
+			return Digest::SHA::hmac_sha256_hex($Td, $self->{secret} );
+		}
+		else {
+			return Digest::SHA::hmac_sha1_hex($Td, $self->{secret} );
+		}
+	}
+}
 
 sub base32enc {
 	my $self = shift;
@@ -245,7 +283,9 @@ sub validate_otp {
 
 		my $T = sprintf("%016x", int($when / $self->{period}) );
 		my $Td = pack('H*', $T);
-		my $hmac = Digest::HMAC_SHA1::hmac_sha1_hex($Td, $self->{secret});
+		
+		my $hmac = $self->hmac($Td);
+		
 		# take the 4 least significant bits (1 hex char) from the encrypted string as an offset
 		my $offset = hex(substr($hmac, -1));
 		# take the 4 bytes (8 hex chars) at the offset (* 2 for hex), and drop the high bit
@@ -310,7 +350,7 @@ __END__
 
 Authen::TOTP - Interface to RFC6238 two factor authentication (2FA)
 
-Version 0.0.2
+Version 0.0.6
 
 =head1 SYNOPSIS
 
@@ -321,7 +361,7 @@ Version 0.0.2
 C<Authen::TOTP> is a simple interface for creating and verifying RFC6238 OTPs
 as used by Google Authenticator, Authy, Duo Mobile etc
 
-It currently passes RFC6238 Test Vectors for SHA1
+It currently passes RFC6238 Test Vectors for SHA1, SHA256, SHA512
 
 =head1 USAGE
 
@@ -363,7 +403,7 @@ It currently passes RFC6238 Test Vectors for SHA1
  my $gen = new Authen::TOTP(
 	 digits 	=>	[6|8],
 	 period		=>	[30|60],
-	 algorithm	=>	"SHA1", #currently only this supported
+	 algorithm	=>	"SHA1", #SHA256 and SHA512 are equally valid
 	 secret		=>	"some_random_stuff",
 	 when		=>	<some_epoch>,
 	 tolerance	=>	0,
@@ -383,7 +423,7 @@ C<30>=> OTP is valid for this many seconds
 
 =item algorithm
 
-C<SHA1>=> only SHA1 currently supported by the module as well as most (?) clients
+C<SHA1>=> supported values are SHA1, SHA256 and SHA512, although most clients only support SHA1 AFAIK
 
 =item secret
 
@@ -418,7 +458,7 @@ Usage:
  $gen->generate_otp(
 	 digits 	=>	[6|8],
 	 period		=>	[30|60],
-	 algorithm	=>	"SHA1", #currently only this supported
+	 algorithm	=>	"SHA1", #SHA256 and SHA512 are equally valid
 	 secret		=>	"some_random_stuff",
 	 issuer		=>	"example.com",
 	 user		=>	"some_identifier",
@@ -437,7 +477,7 @@ Usage:
  $gen->validate_otp(
 	 digits 	=>	[6|8],
 	 period		=>	[30|60],
-	 algorithm	=>	"SHA1", #currently only this supported
+	 algorithm	=>	"SHA1", #SHA256 and SHA512 are equally valid
 	 secret		=>	"the_same_random_stuff_you_used_to_generate_the_TOTP",
 	 when		=>	<epoch_to_use_as_reference>,
 	 tolerance	=>	<try this many iterations before/after when>
@@ -450,17 +490,27 @@ Usage:
 
 =head1 Revision History
 
+ 0.0.6
+	Another pointless adjustment in cpanfile
+ 0.0.5
+	Corrected cpanfile to require either MIME::Base32::XS or MIME::Base32
+	and Digest::SHA or Digest::SHA::PurePerl
+ 0.0.4
+	Added missing test vectors
+ 0.0.3
+	Switched to L<Digest::SHA> in order to support SHA256 and SHA512 as well
  0.0.2
 	Added Digest::HMAC_SHA1 and MIME::Base32 to cpanfiles requires (still
-	getting acquainted with L<Minilla>)
+	getting acquainted with Minilla)
  0.0.1
 	Initial Release
 
 =head1 DEPENDENCIES
 
-L<Digest::HMAC_SHA1>,
-
 one of 
+L<Digest::SHA> or L<Digest::SHA::PurePerl>
+
+and
 L<MIME::Base32::XS> or L<MIME::Base32>
 
 L<Imager::QRCode> if you want to generate QRCodes as well
@@ -473,8 +523,6 @@ L<https://tools.ietf.org/html/rfc6238> for more info on TOTPs
 
 =head1 CAVEATS
 
-Only SHA1 currently supported. It should be trivial to add SHA256 and SHA512, but I didn't bother
-
 Some stuff definitely isn't as efficient as it can be
 
 =head1 BUGS
